@@ -1608,8 +1608,7 @@ dbus_binding_tool_output_glib_client (BaseInfo *info, GIOChannel *channel, gbool
   _dbus_g_type_specialized_builtins_init ();
 
   WRITE_OR_LOSE ("/* Generated by dbus-binding-tool; do not edit! */\n\n");
-  WRITE_OR_LOSE ("#include <glib/gtypes.h>\n");
-  WRITE_OR_LOSE ("#include <glib/gerror.h>\n");
+  WRITE_OR_LOSE ("#include <glib.h>\n");
   WRITE_OR_LOSE ("#include <dbus/dbus-glib.h>\n\n");
   WRITE_OR_LOSE ("G_BEGIN_DECLS\n\n");
 
