@@ -313,6 +313,7 @@ main (int argc, char **argv)
   DBusGProxy *driver;
   DBusGProxy *proxy;
   DBusGProxy *proxy2;
+  DBusGProxy *property_proxy;
   char **name_list;
   guint name_list_len;
   guint i;
@@ -324,7 +325,7 @@ main (int argc, char **argv)
     
   g_type_init ();
 
-  g_log_set_always_fatal (G_LOG_LEVEL_WARNING);
+  g_log_set_always_fatal (G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL);
   
   loop = g_main_loop_new (NULL, FALSE);
 
@@ -456,6 +457,35 @@ main (int argc, char **argv)
   exit_timeout = g_timeout_add (5000, timed_exit, loop);
   g_main_loop_run (loop);
 
+  /* Exercise invalid number/type of return values */
+
+  g_print ("Invalid args; calling Echo\n");
+  if (dbus_g_proxy_call (proxy, "Echo", &error,
+			  G_TYPE_STRING, "my string hello",
+			  G_TYPE_INVALID,
+			  G_TYPE_INVALID))
+    lose ("Unexpected success for invalid Echo return values");
+  g_clear_error (&error);
+
+  g_print ("Invalid args 2; calling Echo\n");
+  if (dbus_g_proxy_call (proxy, "Echo", &error,
+			  G_TYPE_STRING, "my string hello",
+			  G_TYPE_INVALID,
+                          G_TYPE_UINT, &v_UINT32_2,
+			  G_TYPE_INVALID))
+    lose ("Unexpected success for invalid Echo return values");
+  g_clear_error (&error);
+
+  g_print ("Invalid args 3; calling Echo\n");
+  if (dbus_g_proxy_call (proxy, "Echo", &error,
+			  G_TYPE_STRING, "my string hello",
+			  G_TYPE_INVALID,
+                          G_TYPE_STRING, &v_STRING_2,
+                          G_TYPE_UINT, &v_UINT32_2,
+			  G_TYPE_INVALID))
+    lose ("Unexpected success for invalid Echo return values");
+  g_clear_error (&error);
+
   /* Test oneway call and signal handling */
 
   g_print ("Testing Foo emission\n");
@@ -949,6 +979,7 @@ main (int argc, char **argv)
     g_assert (!strcmp ("/org/freedesktop/DBus/GLib/Tests/MyTestObject2",
 		       g_value_get_boxed (g_value_array_get_nth (vals_ret, 1))));
 
+    g_free (val);
     g_value_array_free (vals);
     g_value_array_free (vals_ret);
   }
@@ -1189,6 +1220,7 @@ main (int argc, char **argv)
       lose_gerror ("Failed to send a vairant of array of ints 1, 2 and 3!", error);
 
     g_value_unset (variant);
+    g_free (variant);
   }
 
   for (i=0; i<3; i++)
@@ -1621,9 +1653,69 @@ main (int argc, char **argv)
 
     if (!(found_introspectable && found_myobject && found_properties))
       lose ("Missing interface"); 
+    g_free (node);
   }
   g_free (v_STRING_2);
-  
+
+  /* Properties tests */
+  property_proxy = dbus_g_proxy_new_from_proxy (proxy, DBUS_INTERFACE_PROPERTIES, NULL);
+
+  g_print ("Calling GetProperty (1)\n");
+  {
+    GValue value = {0,};
+    if (!dbus_g_proxy_call (property_proxy, "Get", &error,
+                            G_TYPE_STRING, "org.freedesktop.DBus.GLib.Tests.MyObject",
+                            G_TYPE_STRING, "this_is_a_string", 
+                            G_TYPE_INVALID,
+                            G_TYPE_VALUE, &value, G_TYPE_INVALID))
+      lose_gerror ("Failed to complete GetProperty call", error);
+    g_assert (G_VALUE_HOLDS (&value, G_TYPE_STRING));
+    g_assert (!strcmp (g_value_get_string (&value), ""));
+    g_value_unset (&value);
+  }
+
+  g_print ("Calling SetProperty (1)\n");
+  {
+    GValue value = {0,};
+    g_value_init (&value, G_TYPE_STRING);
+    g_value_set_string (&value, "testing value");
+    if (!dbus_g_proxy_call (property_proxy, "Set", &error,
+                            G_TYPE_STRING, "org.freedesktop.DBus.GLib.Tests.MyObject",
+                            G_TYPE_STRING, "this_is_a_string", 
+                            G_TYPE_VALUE, &value, G_TYPE_INVALID, G_TYPE_INVALID))
+      lose_gerror ("Failed to complete SetProperty call", error);
+    g_value_unset (&value);
+  }
+
+  g_print ("Calling GetProperty (2)\n");
+  {
+    GValue value = {0,};
+    if (!dbus_g_proxy_call (property_proxy, "Get", &error,
+                            G_TYPE_STRING, "org.freedesktop.DBus.GLib.Tests.MyObject",
+                            G_TYPE_STRING, "this_is_a_string", 
+                            G_TYPE_INVALID,
+                            G_TYPE_VALUE, &value, G_TYPE_INVALID))
+      lose_gerror ("Failed to complete GetProperty call", error);
+    g_assert (G_VALUE_HOLDS (&value, G_TYPE_STRING));
+    g_assert (!strcmp (g_value_get_string (&value), "testing value"));
+    g_value_unset (&value);
+  }
+
+  g_print ("Calling GetProperty (3)\n");
+  {
+    GValue value = {0,};
+    if (dbus_g_proxy_call (property_proxy, "Get", &error,
+                            G_TYPE_STRING, "org.freedesktop.DBus.GLib.Tests.MyObject",
+                            G_TYPE_STRING, "SomeUnknownProperty", 
+                            G_TYPE_INVALID,
+                            G_TYPE_VALUE, &value, G_TYPE_INVALID))
+      lose_gerror ("Unexpected success for GetProperty call of unknown property", error);
+
+    g_clear_error (&error);
+  }
+
+  g_object_unref (property_proxy);
+  property_proxy = NULL;
 
   test_terminate_proxy1 = dbus_g_proxy_new_for_name_owner (connection,
                             "org.freedesktop.DBus.GLib.TestService",
